@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: htmlgen.h,v 1.51 2001/03/19 19:27:40 root Exp $
+ * 
  *
- * Copyright (C) 1997-2013 by Dimitri van Heesch.
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -39,19 +39,27 @@ class HtmlCodeGenerator : public CodeOutputInterface
     void writeCodeLink(const char *ref,const char *file,
                        const char *anchor,const char *name,
                        const char *tooltip);
+    void writeTooltip(const char *id, 
+                      const DocLinkInfo &docInfo,
+                      const char *decl,
+                      const char *desc,
+                      const SourceLinkInfo &defInfo,
+                      const SourceLinkInfo &declInfo
+                     );
     void writeLineNumber(const char *,const char *,const char *,int);
     void startCodeLine(bool);
     void endCodeLine();
-    void startCodeAnchor(const char *label);
-    void endCodeAnchor();
     void startFontClass(const char *s);
     void endFontClass();
     void writeCodeAnchor(const char *anchor);
-    void linkableSymbol(int,const char *,Definition *,Definition *);
     void setCurrentDoc(Definition *,const char *,bool) {}
     void addWord(const char *,bool) {}
 
   private:
+    void _writeCodeLink(const char *className,
+                        const char *ref,const char *file,
+                        const char *anchor,const char *name,
+                        const char *tooltip);
     void docify(const char *str);
     bool m_streamSet;
     FTextStream m_t;
@@ -95,23 +103,20 @@ class HtmlGenerator : public OutputGenerator
     { m_codeGen.writeCodeLink(ref,file,anchor,name,tooltip); }
     void writeLineNumber(const char *ref,const char *file,const char *anchor,int lineNumber)
     { m_codeGen.writeLineNumber(ref,file,anchor,lineNumber); }
+    void writeTooltip(const char *id, const DocLinkInfo &docInfo, const char *decl,
+                      const char *desc, const SourceLinkInfo &defInfo, const SourceLinkInfo &declInfo
+                     )
+    { m_codeGen.writeTooltip(id,docInfo,decl,desc,defInfo,declInfo); }
     void startCodeLine(bool hasLineNumbers)
     { m_codeGen.startCodeLine(hasLineNumbers); }
     void endCodeLine()
     { m_codeGen.endCodeLine(); }
-    void startCodeAnchor(const char *label)
-    { m_codeGen.startCodeAnchor(label); }
-    void endCodeAnchor()
-    { m_codeGen.endCodeAnchor(); }
     void startFontClass(const char *s) 
     { m_codeGen.startFontClass(s); }
     void endFontClass() 
     { m_codeGen.endFontClass(); }
     void writeCodeAnchor(const char *anchor) 
     { m_codeGen.writeCodeAnchor(anchor); }
-    void linkableSymbol(int line,const char *symName,
-                        Definition *symDef,Definition *context) 
-    { m_codeGen.linkableSymbol(line,symName,symDef,context); }
     // ---------------------------
 
     void setCurrentDoc(Definition *context,const char *anchor,bool isSourceFile);
@@ -220,7 +225,9 @@ class HtmlGenerator : public OutputGenerator
     void endDescForItem()   { t << "</dd>\n"; }
     void lineBreak(const char *style);
     void writeChar(char c);
-    void startMemberDoc(const char *,const char *,const char *,const char *,bool);
+    void startMemberDoc(const char *clName, const char *memName, 
+                        const char *anchor, const char *title, 
+                        int memCount, int memTotal, bool showInline);
     void endMemberDoc(bool); 
     void startDoxyAnchor(const char *fName,const char *manName,
                          const char *anchor,const char *name,
@@ -265,25 +272,16 @@ class HtmlGenerator : public OutputGenerator
     void startContents();
     void endContents();
     void writeNonBreakableSpace(int);
-    
-    void startDescTable(const char *title)
-    //{ t << "<table border=\"0\" cellspacing=\"2\" cellpadding=\"0\">" << endl; }
-    { t << "<table class=\"fieldtable\">" << endl
-        << "<tr><th colspan=\"2\">" << title << "</th></tr>";
-    }
-    void endDescTable()
-    { t << "</table>" << endl; }
-    void startDescTableTitle()
-    //{ t << "<tr><td valign=\"top\"><em>"; }
-    { t << "<tr><td class=\"fieldname\"><em>"; }
-    void endDescTableTitle()
-    { t << "</em>&nbsp;</td>"; }
-    void startDescTableData()
-    //{ t << "<td>" << endl; }
-    { t << "<td class=\"fielddoc\">" << endl; }
-    void endDescTableData()
-    { t << "</td></tr>" << endl; }
-    
+
+    void startDescTable(const char *title);
+    void endDescTable();
+    void startDescTableRow();
+    void endDescTableRow();
+    void startDescTableTitle();
+    void endDescTableTitle();
+    void startDescTableData();
+    void endDescTableData();
+
     void startDotGraph();
     void endDotGraph(const DotClassGraph &g);
     void startInclDepGraph();
@@ -312,6 +310,7 @@ class HtmlGenerator : public OutputGenerator
     void endParameterName(bool last,bool emptyList,bool closeBracket);
     void startParameterList(bool);
     void endParameterList();
+    virtual void exceptionEntry(const char*,bool);
 
     void startConstraintList(const char *);
     void startConstraintParam();
@@ -322,8 +321,8 @@ class HtmlGenerator : public OutputGenerator
     void endConstraintDocs();
     void endConstraintList();
 
-    void startMemberDocSimple();
-    void endMemberDocSimple();
+    void startMemberDocSimple(bool);
+    void endMemberDocSimple(bool);
     void startInlineMemberType();
     void endInlineMemberType();
     void startInlineMemberName();
